#include <stdio.h>
#include <stdlib.h>

typedef struct _PatientNode
{
  char *name;
  int priority;
} PatientNode;

typedef struct _PatientHeap
{
  PatientNode *array;
  int dim_all;
  int dim_arr;
} PatientHeap;


#define STRING_DIM 101

int heap_new(PatientHeap *heap);
int heap_destroy(PatientHeap *heap);
int heap_larger(PatientHeap *heap);

int heap_insert(PatientHeap *heap, char *name);
int heap_remove_head(char **name, PatientHeap *heap);

int heap_rebalance_up(PatientHeap *heap, int pos);
int heap_rebalance_down(PatientHeap *heap, int pos);

int heap_swap(PatientHeap *heap, int pos, int newpos);

int heap_print_debug(PatientHeap *heap);
int heap_print(PatientHeap *heap);

int nodes_compare(const void *a, const void *b);

#define heap_pos_father(num) (((num)-1)/2)
#define heap_pos_left_child(num) (((num) * 2)+1)
#define heap_pos_right_child(num) (((num) * 2)+2)

#define heap_pos_priority(heap,pos) (((heap).array)[pos].priority)

int main(int argc, char *argv[])
{
  PatientHeap heap;
  char *name_input;
  int input;

  heap_new(&heap);

  while (1)
  {
    scanf("%d", &input);

    if (input == 1)
    {
      name_input = malloc(sizeof(char) * STRING_DIM);
      scanf("%s", name_input);
      heap_insert(&heap, name_input);
      name_input = NULL;
    }
    else if (input == 2)
    {
      char *temp;
      heap_remove_head(&temp, &heap);
      free(temp);
    }
    else if (input == 0)
      break;

  }

  qsort(heap.array, heap.dim_arr, sizeof(PatientNode), nodes_compare);

  heap_print(&heap);

  return 0;
}

int heap_new(PatientHeap *heap)
{
  heap->array = NULL;
  heap->dim_all = 0;
  heap->dim_arr = 0;
  
  return 0;
}

int heap_larger(PatientHeap *heap)
{
  if (heap->dim_all == 0)
  {
    heap->dim_all = 1;
    heap->array = malloc(sizeof(PatientNode));
    return 0;
  }
  else
  {
    heap->dim_all *= 2;
    heap->array = realloc(heap->array, sizeof(PatientNode) * heap->dim_all);
    return 0;
  }
}

int heap_insert(PatientHeap *heap, char *name)
{
  int priority;
  if (heap->dim_arr == heap->dim_all)
    heap_larger(heap);

  (heap->array)[heap->dim_arr].name = name;
  if (heap->dim_arr == 0)
    priority = 0;
  else
    priority = (heap->array)[heap->dim_arr - 1].priority + 1;
    
  (heap->array)[heap->dim_arr].priority = priority;
  heap_rebalance_up(heap, heap->dim_arr);
  (heap->dim_arr)++;

  return 0;
}

int heap_remove_head(char **name, PatientHeap *heap)
{
  if (heap->dim_arr == 0)
    return 1;

  *name = heap->array[0].name;

  heap_swap(heap, 0, heap->dim_arr - 1);
  (heap->dim_arr)--;

  heap_rebalance_down(heap, 0);

  return 0;
}


int heap_rebalance_up(PatientHeap *heap, int pos)
{
  int newpos;

  if (pos == 0)
    return 0;

  newpos=heap_pos_father(pos);
  if (heap_pos_priority(*heap, pos) >= heap_pos_priority(*heap, newpos))
    return 0;

  heap_swap(heap, pos, newpos);

  return heap_rebalance_up(heap, newpos);

}


int heap_rebalance_down(PatientHeap *heap, int pos)
{
  if (pos >= heap->dim_arr - 1)
    return 0;
  else
  {
    int pos_left;
    int pos_right;
    int newpos;

    pos_left = heap_pos_left_child(pos);
    pos_right = heap_pos_right_child(pos);

    if (pos_left >= heap->dim_arr)
      return 0;

    if (pos_right >= heap->dim_arr - 1 ||
        heap_pos_priority(*heap, pos_right) > heap_pos_priority(*heap, pos_left))
      newpos = pos_left;
    else
      newpos = pos_right;

    heap_swap(heap, pos, newpos);

    return heap_rebalance_down(heap, newpos);

  }
}
      
int heap_swap(PatientHeap *heap, int pos, int newpos)
{
  PatientNode temp;

  temp = (heap->array)[newpos];
  (heap->array)[newpos] = (heap->array)[pos];
  (heap->array)[pos] = temp;

  return 0;
}

int heap_print_debug(PatientHeap *heap)
{
  int i;

  for (i = 0; i < heap->dim_arr; i++)
    printf("%d - %s\n", (heap->array)[i].priority,
                        (heap->array)[i].name);
  return 0;
}

int heap_destroy(PatientHeap *heap)
{
  int i;

  for (i = 0; i < heap->dim_arr; i++)
    free(heap->array[i].name);

  free(heap->array);

  return 0;
}

int heap_print(PatientHeap *heap)
{
  int i;

  for (i = 0; i < heap->dim_arr; i++)
    printf("%s\n", heap->array[i]);

  printf("$\n");

  return 0;
}

int nodes_compare(const void *a, const void *b)
{
  PatientNode *node_a;
  PatientNode *node_b;

  node_a=(PatientNode *)a;
  node_b=(PatientNode *)b;

  return strcmp(node_a->name, node_b->name);
}
